from typing import Union

from redis import Redis
import ujson


class BaseStorage:
    """
     An abstract storage class.
     To create your own storage (using, for example, a DB or a file),
      you need to implement all the methods
    """

    def get_user_data(self, user_id: Union[str, int]) -> dict:
        pass

    def set_user_data(self, user_id: Union[str, int], data: dict):
        pass

    def update_user_data(self, user_id: Union[str, int], key: str, value: str):
        pass

    def get_user_state(self, user_id: Union[str, int]) -> str:
        pass

    def set_user_state(self, user_id: Union[str, int], state: str):
        pass


class RedisStorage(Redis, BaseStorage):
    """ Storage based on Redis """

    def __init__(self, host="localhost", port=6379, db=0,
                 username=None, password=None):
        super().__init__(host=host, port=port, db=db,
                         username=username, password=password)

    # DATA

    def get_user_data(self, user_id: Union[str, int]) -> dict:
        if data := self.get(f'{user_id}_data'):
            return ujson.loads(data)
        return {}

    def set_user_data(self, user_id: Union[str, int], data: dict):
        self.set(f'{user_id}_data', ujson.dumps(data))

    def update_user_data(self, user_id: Union[str, int], key: str, value: str):
        data = self.get_user_data(user_id)
        data[key] = value
        self.set_user_data(user_id, data)

    # STATE

    def get_user_state(self, user_id: Union[str, int]) -> str:
        if res := self.get(f'{user_id}_state'):
            return res.decode('utf-8')
        return ''

    def set_user_state(self, user_id: Union[str, int], state):
        self.set(f'{user_id}_state', state)



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="bot-storage",
	version="1.0.0",
	author="Daniil Marchenko, Ivan Romanchenko",
	author_email="vanvanych789@gmail.com",
	description="Storage for bots that allows you to save the states and data of users.",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/IvanRomanchenko/bot-storage",
	packages=["bot_storage", ],
	include_package_data=True,
	classifiers=[
		"Programming Language :: Python :: 3.10",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires=">=3.6",
	install_requires=[
		"redis==4.1.4",
		"ujson==5.1.0",
	]
)
